using System;
using System.Drawing;
using MonoTouch.Foundation;
using MonoTouch.UIKit;
using MonoTouch.AVFoundation;

namespace ScanBarCodes
{
	public partial class ScanBarCodesViewController : UIViewController  
	{
		#region  Member Variables
		AVCaptureSession _session;
		AVCaptureDevice _device;
		AVCaptureDeviceInput _input;
		AVCaptureMetadataOutput _output;
		AVCaptureVideoPreviewLayer _prevLayer;

		UIView _highlightView;
		UILabel _label;
		UIImageView _imageView;

		SimpleCaptureMetadataOutputObjectsDelegate _outputDelegate;
		#endregion


		#region Properties
		public AVCaptureVideoPreviewLayer PreviousLayer {
			get
			{
				return _prevLayer;
			}
		}
		public UIView HighlightView {
			get 
			{
				return _highlightView;
			}
		}
		public UILabel Label {
			get {
				return _label;
			}
		}
		public UIImageView ImageViewer
		{
			get{
				return _imageView;
			}
		}
		public bool PendingRequest {
			get;
			set;
		}
		public bool IsIPhone
		{
			get{
				return UserInterfaceIdiomIsPhone;
			}
		}
		#endregion



		static bool UserInterfaceIdiomIsPhone {
			get { return UIDevice.CurrentDevice.UserInterfaceIdiom == UIUserInterfaceIdiom.Phone; }
		}

		public ScanBarCodesViewController ()
			: base (UserInterfaceIdiomIsPhone ? "ScanBarCodesViewController_iPhone" : "ScanBarCodesViewController_iPad", null)
		{
		}

		public override void DidReceiveMemoryWarning ()
		{
			// Releases the view if it doesn't have a superview.
			base.DidReceiveMemoryWarning ();
			
			// Release any cached data, images, etc that aren't in use.
		}

		public override void ViewDidLoad ()
		{
			base.ViewDidLoad ();
			
			// Perform any additional setup after loading the view, typically from a nib.
			this.PendingRequest = false;

			_imageView = new UIImageView ();
			_imageView.Frame = UserInterfaceIdiomIsPhone ? new RectangleF (0, 0, 60, 60) :  new RectangleF (0, 0, 150, 150);
			_imageView.Layer.BorderWidth = 2;
			_imageView.Layer.BorderColor = UIColor.Red.CGColor;
			_imageView.AutoresizingMask = 
					UIViewAutoresizing.FlexibleRightMargin|
					UIViewAutoresizing.FlexibleBottomMargin;
			this.View.AddSubview (_imageView);

			_highlightView = new UIView ();
			_highlightView.AutoresizingMask = UIViewAutoresizing.FlexibleTopMargin |
				UIViewAutoresizing.FlexibleLeftMargin|
					UIViewAutoresizing.FlexibleRightMargin|
					UIViewAutoresizing.FlexibleBottomMargin;

			_highlightView.Layer.BorderColor = UIColor.Green.CGColor;
			_highlightView.Layer.BorderWidth = 3;
			this.View.AddSubview (_highlightView);

			_label = new UILabel ();
			_label.Frame = new  RectangleF(0, this.View.Bounds.Size.Height - 40, this.View.Bounds.Size.Width, 40);
			_label.AutoresizingMask = UIViewAutoresizing.FlexibleTopMargin;

			_label.BackgroundColor = UIColor.FromWhiteAlpha(0.15f,0.65f);
			_label.TextColor =  UIColor.White;
			_label.TextAlignment = UITextAlignment.Center;
			_label.Text = @"(none)";
			this.View.AddSubview(_label);

			_session = new AVCaptureSession ();
			_device = AVCaptureDevice.DefaultDeviceWithMediaType(AVMediaType.Video);
			NSError error = null;

			_input =    AVCaptureDeviceInput.FromDevice (_device ,out  error);
			if (_input!=null) {
				_session.AddInput(_input);
			} else {
			// NSLog(@"Error: %@", error);
			}
			_outputDelegate= new SimpleCaptureMetadataOutputObjectsDelegate (this);


			_output = new AVCaptureMetadataOutput ();
			_output.SetDelegate(_outputDelegate, MonoTouch.CoreFoundation.DispatchQueue.MainQueue);
			_session.AddOutput(_output);

			_output.MetadataObjectTypes = _output.AvailableMetadataObjectTypes;

			_prevLayer = new AVCaptureVideoPreviewLayer(_session);
			_prevLayer.Frame = this.View.Bounds;
			_prevLayer.VideoGravity = AVLayerVideoGravity.ResizeAspectFill.ToString();
			this.View.Layer.AddSublayer( _prevLayer);

			_session.StartRunning ();

			this.View.BringSubviewToFront (_highlightView);
			this.View.BringSubviewToFront (_label);
			this.View.BringSubviewToFront (_imageView);		
		}
	}
}