using System;
using System.Drawing;
using MonoTouch.Foundation;
using MonoTouch.UIKit;
using MonoTouch.AVFoundation;

namespace ScanBarCodes
{
	public class SimpleCaptureMetadataOutputObjectsDelegate : AVCaptureMetadataOutputObjectsDelegate
	{
		#region Private Members
		ScanBarCodesViewController _parentViewController;
		#endregion

		#region ctor
		public SimpleCaptureMetadataOutputObjectsDelegate(ScanBarCodesViewController viewController)
		{
			_parentViewController = viewController;
		}
		#endregion

		public override void DidOutputMetadataObjects (AVCaptureMetadataOutput captureOutput, AVMetadataObject[] metadataObjects, AVCaptureConnection connection)
		{
			RectangleF highlightViewRect = RectangleF.Empty;
			AVMetadataMachineReadableCodeObject barCodeObject;
			string detectionString = null;

			string[] barCodeTypes =  {
				AVMetadataObject.TypeUPCECode.ToString(),
				AVMetadataObject.TypeCode39Code.ToString(),
				AVMetadataObject.TypeCode39Mod43Code.ToString(),
				AVMetadataObject.TypeEAN13Code.ToString(),
				AVMetadataObject.TypeEAN8Code.ToString(),
				AVMetadataObject.TypeCode93Code.ToString(),
				AVMetadataObject.TypeCode128Code.ToString(),
				AVMetadataObject.TypePDF417Code.ToString(),
				AVMetadataObject.TypeQRCode.ToString(),
				AVMetadataObject.TypeAztecCode.ToString()

			};

			foreach (AVMetadataObject metadata in metadataObjects) {
				foreach (NSString type in barCodeTypes) {
					if ( metadata.Type.ToString() == type)
					{
						barCodeObject = (AVMetadataMachineReadableCodeObject)_parentViewController.PreviousLayer.GetTransformedMetadataObject ((AVMetadataMachineReadableCodeObject)metadata);
						highlightViewRect = barCodeObject.Bounds;
						detectionString = ((AVMetadataMachineReadableCodeObject)metadata).StringValue;
						break;
					}
				}

				if (detectionString != null)
				{
					_parentViewController.Label.Text = detectionString;

					if (!_parentViewController.PendingRequest) {
						_parentViewController.PendingRequest = true;
					
						// Uses open library
						string baseURL = _parentViewController.IsIPhone ? @"http://covers.openlibrary.org/b/ISBN/{0}-S.jpg" :@"http://covers.openlibrary.org/b/ISBN/{0}-M.jpg";

						string address = string.Format (baseURL, detectionString);
		
						CoverUrlConnectionDelegate del = new CoverUrlConnectionDelegate (_parentViewController);

						NSUrlRequest request = new NSUrlRequest (new NSUrl (address));

						NSUrlConnection conn = new NSUrlConnection (request, del);

						conn.Start ();
					}
					break;
				}
				else
				{
					_parentViewController.Label.Text = @"(none)";
				}
			}

			_parentViewController.HighlightView.Frame = highlightViewRect;
		}
	}
}

